from ..cryption import cryption5
import os
from json import loads, dumps

class sessions:

    def __init__(self):
        self.key = "pyrubiIsAPowerfulLibraryForBuildingTheSelfBotInRubika"
        self.crypto = cryption5(self.key)

    def cheack_session(self, session_name):
        return os.path.exists(f'{session_name}.pyrubi')
    
    def session_data(self, session_name):
        return loads(self.crypto.decrypt(open(f'{session_name}.pyrubi', encoding='UTF-8').read()))
        
    def create_session(self, session_name, session_data):
        open(f'{session_name}.pyrubi', 'w', encoding='UTF-8').write(self.crypto.encrypt(dumps(session_data)))


from ..servers import get_server
from urllib3 import PoolManager
from json import dumps, loads
from requests import  post

class maker:
    
    def __init__(self, send_auth, crypto):
        self.auth = send_auth
        self.crypto = crypto
        self.http = PoolManager()
        self.server = get_server('api')
        self.req_clients = {
            'web': {
                'app_name': 'Main',
                'app_version': '4.3.3',
                'platform': 'Web',
                'package': 'web.rubika.ir',
                'lang_code': 'fa'
            },
            'android': {
                'app_name': 'Main',
                'app_version': '3.3.2',
                'platform': 'Android',
                'package': 'ir.resaneh1.iptv',
                'lang_code': 'fa'
            }
        }
        self.raise_info = {
            'INVALID_AUTH': 'The Auth entered is invalid !',
            'NOT_REGISTERED': 'Method input is not registered !',
            'INVALID_INPUT': 'Invalid method input !',
            'TOO_REQUESTS': 'Too much request ! Your account has been suspended.'
        }
        del send_auth

    def method(self, method:str, data:dict, tmp:str=None, api_version:int=6):
        if tmp:
            from ..cryption import cryption6
            self.crypto = cryption6(tmp)
        data = {
            'api_version': str(api_version),
            'tmp_session' if tmp else 'auth': tmp if tmp else self.auth,
            'data_enc': self.crypto.encrypt(
                dumps({
                    'method': method,
                    'input': data,
                    'client': self.req_clients['web']
                })
            )
        }
        if str(api_version) == "6" and not tmp:
            data['sign'] = self.crypto.makeSignFromData(data['data_enc'])

        while True:
            result = self.http.request(
                'POST',
                self.server,
                headers={
                    'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/114.0.0.0 Safari/537.36',
                    'Origin': 'https://web.rubika.ir',
					'Referer': 'https://web.rubika.ir/',
					'Host': self.server.replace('https://','').replace('/','')
                },
                body = dumps(data).encode(),
            )
            result = loads(self.crypto.decrypt(loads(result.data.decode('UTF-8'))['data_enc']))
            if result['status'] == 'OK':
                return result['data']
            elif result['status'] in ['ERROR_GENERIC', 'ERROR_ACTION']:
                raise ConnectionError(self.raise_info[result['status_det']])
            continue

    def _upload(self, url, data, headers):
        while True:
            req = post(url=url, data=data, headers=headers)
            if req.status_code != 200:
                print('This file cannot be uploaded ! Trying to upload again ...')
                continue
            return req.json()
        
    def _download(self, url, headers):
        while True:
            try:
                return self.http.request(method='POST', url=url, headers=headers, preload_content=False)
            except:
                print('This file cannot be downloaded ! Trying to download again ...')
                continue


class message:

    def __init__(self, data):
        self.data = data

    def chat_id(self):
        return self.data['message_updates'][0]['object_guid'] if 'message_updates' in self.data else self.data['object_guid']

    def author_id(self):
        return self.data['message_updates'][0]['message']['author_object_guid'] if 'message_updates' in self.data else self.data['last_message']['author_object_guid']

    def message_id(self):
        return self.data['message_updates'][0]['message_id'] if 'message_updates' in self.data else self.data['last_message']['message_id']

    def reply_to_message_id(self):
        return self.data['message_updates'][0]['message'].get('reply_to_message_id')

    def text(self):
        return str(self.data['message_updates'][0]['message'].get('text') if 'message_updates' in self.data else self.data['last_message'].get('text'))

    def chat_type(self):
        return self.data['message_updates'][0]['type'] if 'message_updates' in self.data else self.data['abs_object']['type']

    def author_type(self):
        return self.data['message_updates'][0]['message']['author_type'] if 'message_updates' in self.data else self.data['last_message']['author_type']

    def message_type(self):
        return self.data['message_updates'][0]['message'].get('type') if 'message_updates' in self.data else self.data['last_message']['type']

    def is_forward(self):
        return 'forwarded_from' in self.data['message_updates'][0]['message'].keys() if 'message_updates' in self.data else None
        
    def forward_type(self):
        return self.data['message_updates'][0]['message']['forwarded_from'].get('type_from') if 'message_updates' in self.data else None
        
    def forward_id(self):
        return self.data['message_updates'][0]['message']['forwarded_from'].get('object_guid') if 'message_updates' in self.data else None
        
    def forward_message_id(self):
        return self.data['message_updates'][0]['message']['forwarded_from'].get('message_id') if 'message_updates' in self.data else None
        
    def is_event(self):
        return 'event_data' in self.data['message_updates'][0]['message'].keys() if 'message_updates' in self.data else self.message_type() == 'Other'

    def is_user_chat(self):
        return self.chat_type() == 'User'

    def is_group_chat(self):
        return self.chat_type() == 'Group'

    def is_channel_chat(self):
        return self.chat_type() == 'Channel'

    def chat_title(self):
        return str(self.data['show_notifications'][0].get('title') if 'message_updates' in self.data else self.data['abs_object'].get('title'))

    def author_title(self):
        return self.data['chat_updates'][0]['chat']['last_message'].get('author_title', self.chat_title()) if 'message_updates' in self.data else self.data['last_message'].get('author_title', self.chat_title())

    def event_type(self):
        return self.data['message_updates'][0]['message']['event_data'].get('type') if 'message_updates' in self.data else None

    def event_id(self):
        return self.data['message_updates'][0]['message']['event_data']['performer_object'].get('object_guid') if 'message_updates' in self.data else None
        
    def count_unseen(self):
        return self.data['chat_updates'][0]['chat'].get('count_unseen', '0') if 'message_updates' in self.data else None


from websocket import create_connection
from json import dumps, loads
from ..servers import get_server
from ..cryption import cryption5
class handler:

    def __init__(self, auth):
        self.crypto = cryption5(auth) if auth else None
        self.auth = auth
        self.hs_data = {
            'api_version': '5',
            'auth': auth,
            'method': 'handShake'
        }
        del auth
    
    def hand_shake(self):
        print('connecting to the web socket...')
        ws = create_connection(get_server('socket'))
        ws.send(dumps(self.hs_data))
        print('connected !')
        while True:
            try:
                recv = loads(ws.recv())
                if recv != {"status":"OK", "status_det":"OK"}:
                    if recv['type'] == 'messenger':
                        yield loads(self.crypto.decrypt(recv['data_enc']))
            except:
                ws.close()
                del ws
                ws = create_connection(self.server)
                print(self.methods.get_me())
                ws.send(dumps(self.hs_data))
                continue